/*****************************************************
Chip type               : ATmega8
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/

#include <mega8.h>


#define A PORTD.4
#define B PORTD.0
#define C PORTB.7
#define D PORTD.6
#define E PORTB.0
#define F PORTD.3
#define G PORTC.5
#define DP PORTD.5
#define A0 PORTC.4
#define A1 PORTD.1
#define A2 PORTD.2
#define A3 PORTB.6

#define mc0_on (PORTB.4 = 0)
#define mc0_off (PORTB.4 = 1)
#define mc1_on (PORTC.3 = 0)
#define mc1_off (PORTC.3 = 1)
#define mc2_on (PORTB.2 = 0)  
#define mc2_off (PORTB.2 = 1) 
#define bcp0_on (PORTB.3 = 1)
#define bcp0_off (PORTB.3 = 0)
#define bcp1_on (PORTB.5 = 1)
#define bcp1_off (PORTB.5 = 0)
#define bcp2_on (PORTB.1 = 1) 
#define bcp2_off (PORTB.1 = 0) 

#define CPU_freq 4000000
#define ADC_itn 16
#define ch_current 300 //mA
#define dch_current 300 //mA

char display_buffer [4], display_counter, dpoint_position;
char sst0, sst1, sst2, sst3, sst4, sst5, sst6, sst7, sst8;   //charging timers
int precharge_timer0, precharge_timer1, precharge_timer2, show_timer, charge_timer;
bit dpoint_bit3, cm_bit;
bit ctimer_trigger0, ctimer_trigger1, ctimer_trigger2;
bit dtimer_trigger0, dtimer_trigger1, dtimer_trigger2;
long cbtimer [3], dbtimer [3];
unsigned int cbcap [3], dbcap [3];
eeprom long eecbtimer [3];
eeprom char stored_battery_state [3];
eeprom char eecm_bit;
unsigned int ADC0_fr, ADC1_fr, ADC2_fr, ADC6_fr, ADC7_fr; 
unsigned char tt0, tt1, tt3;

char adc_timer;
bit mc0, mc1, mc2;

enum battery_state 
{unchecked, checked, installed, discharge, precharge, charge, ready, error} btt0, btt1, btt2;
enum btn_states {b_prs, b_rls, b_hold} btn0;





void ports_init ()
{
PORTB= 0b00000000;
DDRB=  0b11111111;

PORTC=0b00000000;
DDRC=  0b11111000;

PORTD=0b00000000;
DDRD=  0b11111111;
}

void ADC_init ()
{
 ADMUX |= 1<<REFS0 | 1<<REFS1;
 ADCSRA |= 1<<ADEN | 1<<ADIE | 1<<ADPS0 | 1<<ADPS1 |1<<ADPS2; // /128 
}

void ACMP_init ()
{ 
ACSR |= 1<<ACBG | 1<<ACIE | 1<<ACIS1 | 1<<ACIS0;
}

void sleep_init ()
{MCUCR |= 1<<SE;}

void t1_init ()
{
OCR1A = CPU_freq/(4*1024);    // each 0,25s intr
TCCR1B |= 1<<WGM12 | 1<<CS12 | 1<<CS10;  // CTC, OCR1A top, /1024
TIMSK |= 1<<OCIE1A;
}

void scan_key ();
void clear_display ();
void display_data ();
void discharge_animation (char dplace);        
void precharge_animation (char dplace);
void charge_animation (char dplace);       
void convert_data (unsigned int k);
void display_selector ();
void dev_selector ();
void calculate_ccap (char channel);
void calculate_dcap (char channel);
void check_batt ();
unsigned int calculate_rdfc ();

interrupt [TIM1_COMPA] void t1_cmp ()
{
  if (ctimer_trigger0) {cbtimer [0]++;}  //battery capacity timers
  if (ctimer_trigger1) {cbtimer [1]++;}
  if (ctimer_trigger2) {cbtimer [2]++;}
  if (dtimer_trigger0) {dbtimer [0]++;} 
  if (dtimer_trigger1) {dbtimer [1]++;}
  if (dtimer_trigger2) {dbtimer [2]++;}  

  if (tt0) {tt0--;} //animation timers
  if (tt1) {tt1--;}
  if (tt3) {tt3--;} 
  
  if (sst0) {sst0 --;} //check_batt () timers
  if (sst1) {sst1 --;}
  if (sst2) {sst2 --;}
  if (sst3) {sst3 --;}
  if (sst4) {sst4 --;}
  if (sst5) {sst5 --;}
  if (sst6) {sst6 --;}
  if (sst7) {sst7 --;}
  if (sst8) {sst8 --;}
  
  if (precharge_timer0) {precharge_timer0--;} 
  if (precharge_timer1) {precharge_timer1--;}
  if (precharge_timer2) {precharge_timer2--;}
  if (show_timer) {show_timer--;}
  if (charge_timer) {charge_timer--;}  
  
  if (adc_timer) {adc_timer --;}
  
}  

interrupt [ANA_COMP] void acmp_isr()
{
char i;
PORTB=0;
DDRB= 0;
PORTC=0;
DDRC=0;
PORTD=0;
DDRD=0;

for (i=0; i<3; i++){if (eecbtimer [i] != cbtimer [i]) {eecbtimer [i] = cbtimer [i];}}
if (stored_battery_state [0] != btt0) {stored_battery_state [0] = btt0;}
if (stored_battery_state [1] != btt1) {stored_battery_state [1] = btt1;}
if (stored_battery_state [2] != btt2) {stored_battery_state [2] = btt2;}

ports_init ();
}
 
interrupt [ADC_INT] void adc_isr(void)
{
 static char ADC_cnl, ADC_select;  
 char i = 0;
 static unsigned int 
 ADC0_tmp [ADC_itn], ADC1_tmp [ADC_itn], ADC2_tmp [ADC_itn], ADC6_tmp [ADC_itn], ADC7_tmp [ADC_itn];
 
 
if (!ADC_cnl)
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC0_tmp [i] = ADC0_tmp [i+1];}
  ADC0_tmp [(ADC_itn-1)] = ADCW;
  ADC0_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC0_fr += ADC0_tmp [i];}
  ADC0_fr /= ADC_itn;
 }
 else if (ADC_cnl == 1)
 {
  
  for (i=0; i<(ADC_itn-1); i++ ) {ADC1_tmp [i] = ADC1_tmp [i+1];}
  ADC1_tmp [(ADC_itn-1)] = ADCW;
  ADC1_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC1_fr += ADC1_tmp [i];}
  ADC1_fr /= ADC_itn;

 }
 else if (ADC_cnl == 2)
 {

  for (i=0; i<(ADC_itn-1); i++ ) {ADC2_tmp [i] = ADC2_tmp [i+1];}
  ADC2_tmp [(ADC_itn-1)] = ADCW;
  ADC2_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC2_fr += ADC2_tmp [i];}
  ADC2_fr /= ADC_itn;

 } 
 else if (ADC_cnl == 6) 
 {
   for (i=0; i<(ADC_itn-1); i++ ) {ADC6_tmp [i] = ADC6_tmp [i+1];}
  ADC6_tmp [(ADC_itn-1)] = ADCW;
  ADC6_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC6_fr += ADC6_tmp [i];}
  ADC6_fr /= ADC_itn;
 }
 else if (ADC_cnl == 7) 
 {
   for (i=0; i<(ADC_itn-1); i++ ) {ADC7_tmp [i] = ADC7_tmp [i+1];}
  ADC7_tmp [(ADC_itn-1)] = ADCW;
  ADC7_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC7_fr += ADC7_tmp [i];}
  ADC7_fr /= ADC_itn;
 } 

      
    switch (ADC_select)                                                                            
    {
    case 0: mc0_off;ADMUX &= 0b11111000; ADMUX |= 0; ADC_select = 1; ADC_cnl = 0; break;
    case 1: mc1_off; ADMUX &= 0b11111000; ADMUX |= 1; ADC_select = 2; ADC_cnl = 1; mc0 ? mc0_on : mc0_off; break;
    case 2: mc2_off;ADMUX &= 0b11111000; ADMUX |= 2; ADC_select = 6; ADC_cnl = 2;  mc1 ? mc1_on : mc1_off; break;
    case 6: ADMUX &= 0b11111000; ADMUX |= 6; ADC_select = 7; ADC_cnl = 6;  mc2 ? mc2_on : mc2_off;break;
    case 7: ADMUX &= 0b11111000; ADMUX |= 7; ADC_select = 0; ADC_cnl = 7; break;
    default: break;
    }

    {ADCSR |= 1<<ADSC;}
}

void main(void)
{
ports_init ();
t1_init ();
ADC_init ();
//ACMP_init ();
sleep_init ();

cm_bit = 1;
mc0 = mc1 = mc2 = 1;
bcp0_off; bcp1_off; bcp2_off;

//cbtimer [0] = eecbtimer [0];
//cbtimer [1] = eecbtimer [1];
//cbtimer [2] = eecbtimer [2];
//btt0 = stored_battery_state [0];
//btt1 = stored_battery_state [1];
//btt2 = stored_battery_state [2];

#pragma optsize-
WDTCR=0x1F;
WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")
 ADCSR |= 1<<ADSC;

while (1)
      {
      dev_selector ();
      check_batt ();    
      scan_key ();
      
        #asm ("wdr")
      }
}

void convert_data (unsigned int k)
{ char i; for (i = 0; i<4; i++) {display_buffer [i] = k%10; k/=10;} if (display_buffer [3] ==0) display_buffer [3] = 12;}

void discharge_animation (char dplace)        
{
 if (!tt0) {tt0 = 8;}  
 if (tt0 > 6)      {display_buffer [dplace] = 0;}
 else if (tt0 > 4) {display_buffer [dplace] = 25;}
 else if (tt0 > 2) {display_buffer [dplace] = 24;}
 else if (tt0 > 0) {display_buffer [dplace] = 20;}
}

void charge_animation (char dplace)       
{
 if (!tt1) {tt1 = 6;}  
 if (tt1 > 4) {display_buffer [dplace] = 20;}
 else if (tt1 > 2) {display_buffer [dplace] = 19;}
 else if (tt1 > 0) {display_buffer [dplace] = 18;}
}

void precharge_animation (char dplace)
{
 if (!tt3) {tt3 = 4;}  
 if (tt3 > 2) {display_buffer [dplace] = 0;}
 else if (tt3 > 0) {display_buffer [dplace] = 12;}
}

void display_data ()
{
  clear_display ();
  
  if (display_counter<3) {display_counter++;} else {display_counter = 0;}

 switch (display_buffer[display_counter])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break; 
  case 10: A=1; B=1; C=1; D=0; E=0; F=0; G=1; break;     //L
  case 11: A=1; B=0; C=0; D=1; E=0; F=0; G=0; break;     //H
  case 12: A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;     //
  case 13: A=0; B=1; C=1; D=1; E=0; F=0; G=0; break;      //F
  case 14: A=0; B=0; C=1; D=1; E=0; F=0; G=0; break;      //P
  case 15: A=0; B=1; C=1; D=0; E=0; F=0; G=0; break;      //E
  case 16: A=0; B=1; C=1; D=0; E=0; F=0; G=1; break;     //C
  case 17: A=0; B=0; C=0; D=1; E=0; F=0; G=1; break;     //
  case 18: A=0; B=1; C=1; D=0; E=1; F=1; G=0; break;     //_-"
  case 19: A=1; B=1; C=1; D=0; E=1; F=1; G=0; break;     // _-
  case 20: A=1; B=1; C=1; D=0; E=1; F=1; G=1; break;     //_
  case 21: A=0; B=0; C=0; D=1; E=0; F=0; G=0; break;     // A
  case 22: A=1; B=0; C=0; D=0; E=0; F=0; G=0; break;     // ~A
  case 23: A=1; B=1; C=0; D=0; E=0; F=1; G=0; break;     //o
  case 24: A=1; B=1; C=0; D=0; E=0; F=1; G=1; break;     // u 
  case 25: A=1; B=0; C=0; D=0; E=0; F=0; G=1; break;     // U    
  case 26: A=1; B=1; C=1; D=0; E=0; F=0; G=0; break;     // t 
  case 27: A=1; B=1; C=1; D=1; E=0; F=0; G=1; break;     // I
  case 28: A=1; B=1; C=1; D=1; E=0; F=1; G=0; break;     // r
   
  default : A=1; B=1; C=1; D=1; E=1; F=1; G=0; break;     // -
  }
    switch (display_counter)
 {
  case 0: A0 = 1; if (dpoint_position == 0) {DP = 0;} break;
  case 1: A1 = 1; if (dpoint_position == 1) {DP = 0;} break;
  case 2: A2 = 1; if (dpoint_position == 2) {DP = 0;} break;    
  case 3: A3 = 1; if ((dpoint_position == 3)||(dpoint_bit3)) {DP = 0;} break;
  default: break;
 }
}

void clear_display () 
{ 
A=1; B=1; C=1; D=1; E=1; F=1; G=1; DP=1; A0 = 0; A1 = 0; A2 = 0; A3 = 0;
}

void scan_key ()
{
static unsigned char cn0, cn1;

if (ADC6_fr > 700) {cn0 = 0; cn1 = 0; btn0 = b_rls;}
else if (ADC6_fr < 300) {cn0++;} 
if ((cn0 > 50)&&(btn0 != b_hold)) {btn0 = b_prs; cn0 = 0; cn1++;}
if ((cn1 > 20)) {btn0 = b_hold; cn1 = 0;}
}

void display_selector ()
{
char a, b;  
display_buffer [3] = 12;
for (a=0; a < 3; a++) 
{
if (!a) {b = btt0;}
else if (a==1) {b = btt1;}
else if (a==2) {b = btt2;}

switch (b)
{
 case unchecked:  display_buffer [a] = 16; break; //C
 case checked:  display_buffer [a] = 25; break;  //U
 case installed: display_buffer [a] = 1; break; 
 //case removed: display_buffer [a] = 28; break;
 case discharge: discharge_animation (a); break;
 case precharge: precharge_animation (a);  break;
 case charge: charge_animation (a); break; 
 case ready: display_buffer [a] = 13;  break;
 case error: display_buffer [a] = 15; break;
 default: display_buffer [a] = 28; break; 
}
}

}
//--------------------------------------------------------------------------------------------------------
void dev_selector ()
{
 char tmp;
 static unsigned int rdfc_tmp;
 static bit trigger0, trigger1, trigger2, trigger3, trigger4;
 static enum display_modes 
 {cc0, cc1, cc2, dc0, dc1, dc2, rdfc, normal_operation, cm} d_mode = rdfc;    
                                        // ch_capacity, disch_capacity, redef_capacity, ch_method
    
    if (btn0 == b_rls) {trigger0 = 0, trigger3 = 0;} 

    if ((btn0 == b_hold)&&(!trigger3)) {d_mode = cm; trigger3 = 1; trigger2 = 1;}                                                                            
    if (d_mode == cm) {if ((btn0 == b_prs)&&(!trigger3)) {cm_bit = ~cm_bit;  trigger3 = 1; show_timer = 40;}}
    
    if ((btn0 == b_prs)&&(!trigger0)&&(!trigger2)) 
    { trigger0 = 1; if (d_mode < normal_operation) {d_mode++;}  else {d_mode = cc0;} trigger4 = 0;}

    if ((rdfc_tmp < (ADC7_fr - 50))||(rdfc_tmp > (ADC7_fr + 50))) { d_mode = rdfc; rdfc_tmp = ADC7_fr; }
  

 dpoint_position = 5; dpoint_bit3 = 0;
 for (tmp = 0; tmp < 4; tmp++) {display_buffer [tmp] = 12;}

 switch (d_mode)
 {
  case cc0:   calculate_ccap (0); convert_data (cbcap [0]);  dpoint_position = 0; display_data ();
  break; 
  case cc1:   calculate_ccap (1); convert_data (cbcap [1]);  dpoint_position = 1; display_data ();
  break;
  case cc2:   calculate_ccap (2); convert_data (cbcap [2]);  dpoint_position = 2; display_data ();
  break;  
  case dc0:   calculate_dcap (0); convert_data (dbcap [0]); dpoint_position = 0; dpoint_bit3 = 1; display_data ();
  break;
  case dc1:   calculate_dcap (1); convert_data (dbcap [1]); dpoint_position = 1; dpoint_bit3 = 1; display_data ();
  break;
  case dc2:   calculate_dcap (2); convert_data (dbcap [2]); dpoint_position = 2; dpoint_bit3 = 1; display_data ();
  
  break;    
  case rdfc:  
  if (!trigger1){show_timer = 20; trigger1 = 1;}  
  if (show_timer) {convert_data (calculate_rdfc ()); display_data ();}
  else {d_mode = normal_operation; trigger1 = 0;}
  break; 
  
  case cm:   
                 if (!trigger1){show_timer = 40; trigger1 = 1;}
                 if (!show_timer) {d_mode = normal_operation; trigger1 = 0; trigger2 = 0;} 
                 if (cm_bit) {display_buffer [3] = 23; display_buffer [2] = 25; display_buffer [1] = 12; display_buffer [0] = 12; display_data ();}   //oU
                 else {display_buffer [3] = 26; display_buffer [2] = 28; display_buffer [1] = 12; display_buffer [0] = 12; display_data ();}   //tr
  break; 
  
  case normal_operation: display_selector (); display_data ();
  break;
  default: break;   
 }
}
//--------------------------------------------------------------------------------------------------------                                  
void calculate_ccap (char channel)
{ cbcap [channel] = cbtimer [channel] * ch_current /(4*60*60);}

void calculate_dcap (char channel)
{ dbcap [channel] = dbtimer [channel] * dch_current /(4*60*60);}
                                                     
unsigned int calculate_rdfc ()
{
 unsigned int a7;
 a7 = (long) ADC7_fr *6000/1024;
 return a7;
}
                                                 
void check_batt ()       
{
static char a, b;
char i;
static bit trigger0, trigger1, trigger2;
static int izmu0 [16], izmu1 [16], izmu2 [16]; //one minutes V value arrays for dV detector
static char ccd0, ccd1, ccd2;  // coincidence counter


     for (a=0; a < 3; a++)  
    {
    if (!a) {b=btt0;}
    else if (a == 1) {b=btt1;}
    else {b=btt2;}

switch (b)
{
 case unchecked: 
 { 
 if (!a)  
 {   
      cbtimer [0] = dbtimer [0] = 0; for (i=0; i <16; i++) {izmu0 [i] = 0;}
      if ((!sst0)&&(!trigger0)) {mc0=0; mc0_off; bcp0_on; sst0 = 16; trigger0 = 1;}      
      if ((ADC0_fr>((30*1024)/256))&&(!sst0)&&(trigger0)) {btt0 = installed; bcp0_off; trigger0 = 0;}
        else if ((!sst0)&&(trigger0)) {btt0 = precharge; bcp0_off; trigger0 = 0;}
 }
 else if (a == 1) 
 {   
      cbtimer [1] = dbtimer [1] = 0; for (i=0; i <16; i++) {izmu1 [i] = 0;}
      if ((!sst1)&&(!trigger1)) {mc1=0; mc1_off;  bcp1_on; sst1 = 16; trigger1 = 1;}      
      if ((ADC1_fr>((30*1024)/256))&&(!sst1)&&(trigger1)) {btt1 = installed; bcp1_off; trigger1 = 0;}
        else if ((!sst1)&&(trigger1)) {btt1 = precharge; bcp1_off; trigger1 = 0;} 
 }
 else if (a == 2) 
 {
      cbtimer [2] = dbtimer [2] = 0; for (i=0; i <16; i++) {izmu2 [i] = 0;}
      if ((!sst2)&&(!trigger2)) {mc2=0; mc2_off; bcp2_on; sst2 = 16; trigger2 = 1;}      
      if ((ADC2_fr>((30*1024)/256))&&(!sst2)&&(trigger2)) {btt2 = installed; bcp2_off; trigger2 = 0;}
        else if ((!sst2)&&(trigger2)) {btt2 = precharge; bcp2_off; trigger2 = 0;}
 }
 break;
 }   
 case installed:
 {
 if (!a)
 {
 if (ADC0_fr<((100*1024)/256)) {btt0 = charge;}
 else if (ADC0_fr<((135*1024)/256)) {btt0 = discharge;}
 else {btt0 = ready;}    
 }  
 else if (a == 1) 
 {
 if (ADC1_fr<((100*1024)/256)) {btt1 = charge;}
 else if (ADC1_fr<((135*1024)/256)) {btt1 = discharge;}
 else {btt1 = ready;}
 }
 else if (a == 2) 
 {
 if (ADC2_fr<((100*1024)/256)) {btt2 = charge;}
 else if (ADC2_fr<((135*1024)/256)) {btt2 = discharge;}
 else {btt2 = ready;}
 }    
 break;
 }   
 case discharge: 
 {
 if (!a)  
 {
 if (ADC0_fr > ((100*1024)/256)) {mc0=0; mc0_off;  bcp0_on; dtimer_trigger0 = 1;} 
 else {bcp0_off; dtimer_trigger0 = 0; btt0 = charge;} 
 }
 else if (a == 1) 
 {
 if (ADC1_fr > ((100*1024)/256)) {mc1=0; mc1_off;  bcp1_on; dtimer_trigger1 = 1;} 
 else {bcp1_off; dtimer_trigger1 = 0; btt1 = charge;}
 }
 else if (a == 2) 
 {
 if (ADC2_fr > ((100*1024)/256)) {mc2=0; mc2_off; bcp2_on; dtimer_trigger2 = 1;} 
 else {bcp2_off; dtimer_trigger2 = 0; btt2 = charge;} 
 }   
 break;
 }   
 case precharge:
 {
 if (!a)  
 {
  if (!trigger0) {trigger0 = 1; precharge_timer0 = 250;}
  if (precharge_timer0) { mc0=1;  bcp0_off;} 
  else {mc0=0; mc0_off;  btt0 = checked; trigger0 = 0;}
 }
 else if (a == 1) 
 {
  if (!trigger1) {trigger1 = 1; precharge_timer1 = 250;}
  if (precharge_timer1) {mc1=1;  bcp1_off;} 
  else {mc1=0; mc1_off;  btt1 = checked; trigger1 = 0;} 
 }
 else if (a == 2) 
 {
  if (!trigger2) {trigger2 = 1; precharge_timer2 = 250;}
  if (precharge_timer2) {mc2=1;  bcp2_off;} 
  else {mc2=0; mc2_off; btt2 = checked; trigger2 = 0;} 
 }    
 break; 
 }
 case checked: 
 {
 if (!a)  
 {
   if (!trigger0) {mc0=0; mc0_off;  bcp0_on; trigger0 = 1;}      
   else if ((ADC0_fr>((50*1024)/256))&&((ADC0_fr<((100*1024)/256)))) {btt0 = charge; bcp0_off; trigger0 = 0;}
   else if (ADC0_fr>((99*1024)/256)) {btt0 = discharge; bcp0_off; trigger0 = 0;}
   else  {btt0 = error ; bcp0_off; trigger0 = 0;}
 }
 else if (a == 1) 
 {
   if (!trigger1) {mc1=0; mc1_off;  bcp1_on; trigger1 = 1;}      
   else if ((ADC1_fr>((50*1024)/256))&&((ADC1_fr<((100*1024)/256)))) {btt1 = charge; bcp1_off; trigger1 = 0;}
   else if (ADC1_fr>((99*1024)/256)) {btt1 = discharge; bcp1_off; trigger1 = 0;}
   else  {btt1 = error ; bcp1_off; trigger1 = 0;} 
 }
 else if (a == 2) 
 {
   if (!trigger2) {mc2=0; mc2_off; bcp2_on; trigger2 = 1;}      
   else if ((ADC2_fr>((50*1024)/256))&&((ADC2_fr<((100*1024)/256)))) {btt2 = charge; bcp2_off; trigger2 = 0;}
   else if (ADC2_fr>((99*1024)/256)) {btt2 = discharge; bcp2_off; trigger2 = 0;}
   else  {btt2 = error ; bcp2_off; trigger2 = 0;} 
 }
 break;
 }
 case charge:            
 if (!a)  
 { 
 bcp0_off; ctimer_trigger0 = 1; mc0=1;
 
 
  if (cm_bit)  //dU +timer   
 { 
  if (!sst0) //one minutes check
  {
        sst0 = 240;
        for (i=0; i <15; i++) {izmu0 [i] = izmu0 [i+1];}
        izmu0 [15] = ADC0_fr;
        for(i=0; i<15; i++){ if(izmu0 [i] >= izmu0 [15]) {ccd0++;}  else if (ccd0) {ccd0--;} } //dV detection
        calculate_ccap (0);
        if ((ccd0 > 10)||(cbcap [0] > (calculate_rdfc ()))) 
        {mc0=0; mc0_off;  ctimer_trigger0 = 0; btt0 = ready; ccd0 = 0;}
        if ((ADC0_fr > (200*1024/256))||(ADC0_fr < (60*1024/256)))  {mc0=0; mc0_off;  ctimer_trigger0 = 0; btt0 = error;}
        
   }
 }     
 else       //only timer
 {
  if ((ADC0_fr > (200*1024/256))||(ADC0_fr < (60*1024/256))) {mc0=0; mc0_off;  ctimer_trigger0 = 0; btt0 = error;} 
  calculate_ccap (0);
  if (cbcap [0] > (calculate_rdfc ())) {mc0=0; mc0_off;  ctimer_trigger0 = 0; btt0 = ready;}
 }
 
 }
 else if (a == 1) 
 { 
 bcp1_off; ctimer_trigger1 = 1; mc1=1;
 
 
  if (cm_bit)  //dU +timer 
 { 
  if (!sst1) //one minutes check
  {
        sst1 = 240;
        for (i=0; i <15; i++) {izmu1 [i] = izmu1 [i+1];}
        izmu1 [15] = ADC1_fr;
        for(i=0; i<15; i++){ if(izmu1 [i] >= izmu1 [15]) {ccd1++;}  else if (ccd1) {ccd1--;} } //dV detection
        calculate_ccap (1);
        if ((ccd1 > 10)||(cbcap [1] > (calculate_rdfc ()))) 
        {mc1=0; mc1_off;  ctimer_trigger1 = 0; btt1 = ready; ccd1 = 0;}
        if ((ADC1_fr > (200*1024/256))||(ADC1_fr < (60*1024/256)))  {mc1=0; mc1_off;  ctimer_trigger1 = 0; btt1 = error;}
        
   }
 }     
 else       //only timer
 {
  if ((ADC1_fr > (200*1024/256))||(ADC1_fr < (60*1024/256))) {mc1=0; mc1_off;  ctimer_trigger1 = 0; btt1 = error;} 
  calculate_ccap (1);
  if (cbcap [1] > (calculate_rdfc ())) {mc1=0; mc1_off;  ctimer_trigger1 = 0; btt1 = ready;}
 }
 
 }
 else if (a == 2) 
 { 
 bcp2_off; ctimer_trigger2 = 1; mc2=1;
 
 
  if (cm_bit)  //dU +timer   
 { 
  if (!sst2) //one minutes check
  {
        sst2 = 240;
        for (i=0; i <15; i++) {izmu2 [i] = izmu2 [i+1];}
        izmu2 [15] = ADC2_fr;
        for(i=0; i<15; i++){ if(izmu2 [i] >= izmu2 [15]) {ccd2++;}  else if (ccd2) {ccd2--;} } //dV detection
        calculate_ccap (2);
        if ((ccd2 > 10)||(cbcap [2] > (calculate_rdfc ()))) 
        {mc2=0; mc2_off;  ctimer_trigger2 = 0; btt2 = ready; ccd2 = 0;}
        if ((ADC2_fr > (200*1024/256))||(ADC2_fr < (60*1024/256)))  {mc2=0; mc2_off;  ctimer_trigger2 = 0; btt2 = error;}
        
   }
 }     
 else       //only timer
 {
  if ((ADC2_fr > (200*1024/256))||(ADC2_fr < (60*1024/256))) {mc2=0; mc2_off;  ctimer_trigger2 = 0; btt2 = error;} 
  calculate_ccap (2);
  if (cbcap [2] > (calculate_rdfc ())) {mc2=0; mc2_off;  ctimer_trigger2 = 0; btt2 = ready;}
 }
 
 }  
 break;
    
 case ready:
 if (!a)  
 {
 mc0=0; mc0_off;  bcp0_off; 
 if (ADC0_fr < (100*1024/256)) {btt0 = unchecked;}
 }
 else if (a == 1) 
 {
  mc1=0; mc1_off;  bcp1_off; 
 if (ADC1_fr < (100*1024/256)) {btt1 = unchecked;}
 }
 else if (a == 2) 
 {
 mc2=0; mc2_off; bcp2_off; 
 if (ADC2_fr < (100*1024/256)) {btt2 = unchecked;}
 }    
 break; 
 
 case error:
 if (!a)  
 {
 mc0=0; mc0_off;  bcp0_off; 
 if ((ADC0_fr > (50*1024/256))&&(ADC0_fr < (150*1024/256))) {btt0 = unchecked;}
 }
 else if (a == 1) 
 {
 mc1=0; mc1_off;  bcp1_off; 
 if ((ADC1_fr > (50*1024/256))&&(ADC1_fr < (150*1024/256))) {btt1 = unchecked;}
 }
 else if (a == 2) 
 {
 mc2=0; mc2_off; bcp2_off; 
 if ((ADC2_fr > (50*1024/256))&&(ADC2_fr < (150*1024/256))) {btt2 = unchecked;}
 }    
 break;
    
 default: break; 
}
}
    }

  

